library cmapmarker;
import 'package:flare_dart/math/mat2d.dart';
import 'package:flare_flutter/flare.dart';
import 'package:flare_flutter/flare_controller.dart';
import 'package:flutter/material.dart';
import 'package:flare_flutter/flare_actor.dart';

class CMapMarker extends StatefulWidget {
  final double width;
  final double height;
  final Color accentColor;
  final Color insideColor;
  final Color outerStrokeColor;
  final bool animate;

  const CMapMarker({
    Key key,
    this.width = 22,
    this.height = 35,
    this.accentColor = Colors.green,
    this.insideColor = Colors.white,
    this.animate = false,
    this.outerStrokeColor = Colors.white,
  }) : super(key: key);
  @override
  _CMapMarkerState createState() => _CMapMarkerState();
}

class _CMapMarkerState extends State<CMapMarker> with FlareController {
  bool hasColorChanges = false;

  @override
  void didUpdateWidget(CMapMarker oldWidget) {
    super.didUpdateWidget(oldWidget);
    if (widget.accentColor != oldWidget.accentColor ||
        widget.insideColor != oldWidget.insideColor ||
        widget.outerStrokeColor != oldWidget.outerStrokeColor) {
      hasColorChanges = true;
      isActive.value = true;
    }
  }

  @override
  void initState() {
    super.initState();
    isActive.value = true;
  }

  @override
  Widget build(BuildContext context) {
    return Container(
      width: widget.width,
      height: widget.height,
      child: FlareActor(
        "flare/map_marker.flr",
        animation: widget.animate ? "jump" : "idle",
        shouldClip: false,
        controller: this,
      ),
    );
  }

  FlutterColorFill _accentFill;
  FlutterColorFill _insideFill;
  FlutterColorStroke _outerStroke;

  @override
  bool advance(FlutterActorArtboard artboard, double elapsed) {
    if (hasColorChanges) {
      _accentFill?.uiColor = widget.accentColor;
      _insideFill?.uiColor = widget.insideColor;
      _outerStroke?.uiColor = widget.outerStrokeColor;
      hasColorChanges = false;
      debugPrint("Changing shape colors.");
    }
    return false;
  }

  @override
  void initialize(FlutterActorArtboard artboard) {
    FlutterActorShape baseShape = artboard.getNode("markerBase");
    _accentFill = baseShape?.fill as FlutterColorFill;
    _outerStroke = baseShape?.stroke as FlutterColorStroke;
    FlutterActorShape insideShape = artboard.getNode("markerEllipse");
    _insideFill = insideShape?.fill as FlutterColorFill;
  }

  @override
  void setViewTransform(Mat2D viewTransform) {}
}
